@@ -1,12 +1,17 @@
 import os
+import re
 import six
+import sys
+import gzip
 import shutil
+import logging
 from django.conf import settings
 from optparse import make_option
 from django.core import management
 from django.core.urlresolvers import get_callable
 from django.core.exceptions import ViewDoesNotExist
 from django.core.management.base import BaseCommand, CommandError
+logger = logging.getLogger(__name__)
 
 
 custom_options = (
@@ -43,11 +48,67 @@ settings.py or provide it with --build-dir"
     views_unconfig_msg = "Bakery views unconfigured. Set BAKERY_VIEWS in \
 settings.py or provide a list as arguments."
 
+    def copytree_and_gzip(self, source_dir, target_dir):
+        """
+        Copies the provided source directory to the provided target directory
+        and gzips JavaScript, CSS and HTML files along the way.
+        """
+        # regex to match against. CSS, JS, JSON, HTML files
+        gzip_file_match = getattr(
+            settings,
+            'GZIP_FILE_MATCH',
+            '(\.html|\.xml|\.css|\.js|\.json)$'
+        )
+        pattern = re.compile(gzip_file_match)
+
+        # Walk through the source directory...
+        for (dirpath, dirnames, filenames) in os.walk(source_dir):
+
+            # And for each file...
+            for filename in filenames:
+
+                # ... figure out the path to the file...
+                og_file = os.path.join(dirpath, filename)
+
+                # And then where we want to copy it to.
+                rel_path = os.path.relpath(dirpath, source_dir)
+                dest_path = os.path.join(target_dir, rel_path)
+                if not os.path.exists(dest_path):
+                    os.makedirs(dest_path)
+
+                # If it isn't a file want to gzip...
+                if not pattern.search(filename):
+                    # just copy it to the target.
+                    shutil.copy(og_file, dest_path)
+
+                # If it is one we want to gzip...
+                else:
+                    # ... let the world know...
+                    logger.debug("Gzipping %s" % filename)
+                    if self.verbosity > 1:
+                        six.print_("Gzipping %s" % filename)
+
+                    # ... create the new file in the build directory ...
+                    f_in = open(og_file, 'rb')
+                    f_name = os.path.join(dest_path, filename)
+
+                    # ... copy the file to gzip compressed output ...
+                    if float(sys.version[:3]) >= 2.7:
+                        f_out = gzip.GzipFile(f_name, 'wb', mtime=0)
+                    else:
+                        f_out = gzip.GzipFile(f_name, 'wb')
+
+                    # ... and shut it down.
+                    f_out.writelines(f_in)
+                    f_out.close()
+                    f_in.close()
+
     def handle(self, *args, **options):
         """
         Making it happen.
         """
         self.verbosity = int(options.get('verbosity'))
+        logger.info("Build started")
 
         # Figure out what build directory to use
         if options.get("build_dir"):
@@ -59,6 +120,7 @@ settings.py or provide a list as arguments."
             self.build_dir = settings.BUILD_DIR
 
         # Destroy the build directory, if it exists
+        logger.debug("Creating %s" % self.build_dir)
         if self.verbosity > 1:
             six.print_("Creating build directory")
         if os.path.exists(self.build_dir):
@@ -69,16 +131,24 @@ settings.py or provide a list as arguments."
 
         # Build up static files
         if not options.get("skip_static"):
+            logger.debug("Building static directory")
             if self.verbosity > 1:
-                six.print_("Creating static directory")
+                six.print_("Building static directory")
+
             management.call_command(
                 "collectstatic",
                 interactive=False,
                 verbosity=0
             )
             target_dir = os.path.join(self.build_dir, settings.STATIC_URL[1:])
+
             if os.path.exists(settings.STATIC_ROOT) and settings.STATIC_URL:
-                shutil.copytree(settings.STATIC_ROOT, target_dir)
+                if getattr(settings, 'BAKERY_GZIP', False):
+                    self.copytree_and_gzip(settings.STATIC_ROOT, target_dir)
+                # if gzip isn't enabled, just copy the tree straight over
+                else:
+                    shutil.copytree(settings.STATIC_ROOT, target_dir)
+
             # If they exist in the static directory, copy the robots.txt
             # and favicon.ico files down to the root so they will work
             # on the live website.
@@ -96,8 +166,10 @@ settings.py or provide a list as arguments."
                     'favicon.ico',
                     )
                 )
+
         # Build the media directory
         if not options.get("skip_media"):
+            logger.debug("Building static directory")
             if self.verbosity > 1:
                 six.print_("Building media directory")
             if os.path.exists(settings.MEDIA_ROOT) and settings.MEDIA_URL:
@@ -116,6 +188,7 @@ settings.py or provide a list as arguments."
 
         # Then loop through and run them all
         for view_str in view_list:
+            logger.debug("Building %s" % view_str)
             if self.verbosity > 1:
                 six.print_("Building %s" % view_str)
             try:
