@@ -4,6 +4,8 @@ for building flat files.
 """
 import os
 import six
+import sys
+import gzip
 import shutil
 import logging
 from django.conf import settings
@@ -12,11 +14,50 @@ from django.views.generic import TemplateView, DetailView, ListView
 logger = logging.getLogger(__name__)
 
 
-class BuildableTemplateView(TemplateView):
+class BuildableMixin(object):
+    """
+    Common methods we will use in buildable views.
+    """
+    def build_file(self, path, html):
+        if getattr(settings, 'BAKERY_GZIP', False):
+            self.gzip_file(path, html)
+        else:
+            self.write_file(path, html)
+
+    def write_file(self, path, html):
+        """
+        Writes out the provided HTML to the provided path.
+        """
+        logger.debug("Building HTML file to %s" % path)
+        outfile = open(path, 'wb')
+        outfile.write(six.binary_type(html))
+        outfile.close()
+
+    def gzip_file(self, path, html):
+        """
+        Zips up the provided HTML as a companion for the provided path.
+
+        Intended to take advantage of the peculiarities of
+        Amazon S3's GZIP service.
+
+        mtime, an option that writes a timestamp to the output file
+        is set to 0, to avoid having s3cmd do unnecessary uploads because
+        of differences in the timestamp
+        """
+        logger.debug("Building gzipped HTML file to %s" % path)
+        if float(sys.version[:3]) > 2.7:
+            outfile = gzip.GzipFile(path, 'wb', mtime=0)
+        else:
+            outfile = gzip.GzipFile(path, 'wb')
+        outfile.write(six.binary_type(html))
+        outfile.close()
+
+
+class BuildableTemplateView(TemplateView, BuildableMixin):
     """
     Renders and builds a simple template.
 
-    When inherited, the child class should include the following attribuntes:
+    When inherited, the child class should include the following attributes.
 
         build_path:
             The target location of the built file in the BUILD_DIR.
@@ -41,12 +82,10 @@ class BuildableTemplateView(TemplateView):
             dirname = os.path.join(settings.BUILD_DIR, dirname)
             os.path.exists(dirname) or os.makedirs(dirname)
         # Write out the data
-        outfile = open(path, 'wb')
-        outfile.write(six.binary_type(html))
-        outfile.close()
+        self.build_file(path, html)
 
 
-class BuildableListView(ListView):
+class BuildableListView(ListView, BuildableMixin):
     """
     Render and builds a page about a list of objects.
 
@@ -85,12 +124,10 @@ class BuildableListView(ListView):
         html = self.get(self.request).render().content
         # Write it out to the appointed flat file
         path = os.path.join(settings.BUILD_DIR, self.build_path)
-        outfile = open(path, 'wb')
-        outfile.write(six.binary_type(html))
-        outfile.close()
+        self.build_file(path, html)
 
 
-class BuildableDetailView(DetailView):
+class BuildableDetailView(DetailView, BuildableMixin):
     """
     Render and build a "detail" view of an object.
 
@@ -107,11 +144,6 @@ class BuildableDetailView(DetailView):
     def build_method(self):
         return self.build_queryset
 
-    def write(self, path, data):
-        outfile = open(path, 'wb')
-        outfile.write(six.binary_type(data))
-        outfile.close()
-
     def get_url(self, obj):
         """
         The URL at which the detail page should appear.
@@ -146,10 +178,9 @@ class BuildableDetailView(DetailView):
         logger.debug("Building %s" % obj)
         self.request = RequestFactory().get(self.get_url(obj))
         self.set_kwargs(obj)
-        self.write(
-            self.get_build_path(obj),
-            self.get_html()
-        )
+        path = self.get_build_path(obj)
+        html = self.get_html()
+        self.build_file(path, html)
 
     def build_queryset(self):
         [self.build_object(o) for o in self.get_queryset().all()]
